<?php
/**
 * Customers Export
 *
 * This class handles customer export
 *
 * @package FES
 * @subpackage Reports
 * @since 2.3.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// Exit if EDD Export class doesn't exist
if ( ! class_exists( 'EDD_Export' ) ){
	exit;
}

/**
 * FES Customers Export Class
 *
 * This class extends the EDD_Export class
 * to allow for CSV exports of a vendor's
 * customers.
 *
 * @since 2.3.0
 * @access public
 */
class FES_Customers_Export extends EDD_Export {

	/**
	 * The type of export.
	 *
	 * @since 2.3.0
	 * @access public
	 * @var string $export_type Export type. Used for export-type specific filters/actions.
	 */
	public $export_type = 'customers';

	/**
	 * Set the export headers.
	 *
	 * Sets the headers for PHP for the CSV export.
	 *
	 * @since 2.3.0
	 * @access public
	 *
	 * @return void
	 */
	public function headers() {
		ignore_user_abort( true );

		if ( ! edd_is_func_disabled( 'set_time_limit' ) && ! ini_get( 'safe_mode' ) ) {
			set_time_limit( 0 );
		}

		$extra = '';

		if ( ! empty( $_POST['edd_export_download'] ) ) {
			$extra = sanitize_title( get_the_title( absint( $_POST['edd_export_download'] ) ) ) . '-';
		}

		nocache_headers();
		header( 'Content-Type: text/csv; charset=utf-8' );
		header( 'Content-Disposition: attachment; filename=' . apply_filters( 'edd_customers_export_filename', 'edd-export-' . $extra . $this->export_type . '-' . date( 'm-d-Y' ) ) . '.csv' );
		header( "Expires: 0" );
	}

	/**
	 * Sets the CSV columns.
	 *
	 * Sets the CSV columns for the CSV export.
	 *
	 * @since 2.3.0
	 * @access public
	 *
	 * @return array All the columns.
	 */
	public function csv_cols() {
		if ( ! empty( $_POST['edd_export_download'] ) ) {
			$cols = array(
				'product_id'     => sprintf( _x( '%s ID', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'product_title'  => sprintf( _x( '%s Title', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'option_id'      => sprintf( _x( '%s Option ID', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'option_title'   => sprintf( _x( '%s Option Title', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'name'           => __( 'Name', 'edd_fes' ),
				'email'          => __( 'Email', 'edd_fes' ),
				'date'           => __( 'Date Purchased', 'edd_fes' )
			);
		} else {

			$cols = array(
				'product_id'     => sprintf( _x( '%s ID', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'product_title'  => sprintf( _x( '%s Title', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'option_id'      => sprintf( _x( '%s Option ID', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
				'option_title'   => sprintf( _x( '%s Option Title', 'FES singular uppercase setting for download', 'edd_fes' ), EDD_FES()->helper->get_product_constant_name( $plural = false, $uppercase = true ) ),
			);

			if ( 'emails' != $_POST['fes_export_option'] ) {
				$cols['name'] = __( 'Name',   'edd_fes' );
			}

			$cols['email'] = __( 'Email',   'edd_fes' );

			if ( 'full' == $_POST['fes_export_option'] ) {
				$cols['purchases'] = __( 'Total Purchases',   'edd_fes' );
				$cols['amount']    = __( 'Total Purchased', 'edd_fes' ) . ' (' . html_entity_decode( edd_currency_filter( '' ) ) . ')';
			}

		}

		return $cols;
	}

	/**
	 * Get the Export Data.
	 *
	 * Gets the data used in the CSV export of the
	 * customers.
	 *
	 * @access public
	 * @since 2.3.0
	 *
	 * @return array The data for the CSV file
	 */
	public function get_data() {
		$data = array();
		if ( function_exists( 'edd_get_order_items' ) ) {
			$data = $this->get_data_30();
		} else {
			$data = $this->get_data_29();
		}

		/**
		 * Export CSV data.
		 *
		 * Allows for the data for the CSV export
		 * to be altered.
		 *
		 * @since 2.3.0
		 *
		 * @param array $data Data for the export.
		 */
		$data = apply_filters( 'fes_export_get_data', $data );

		/**
		 * Customer CSV data.
		 *
		 * Allows for the data for the CSV customer export
		 * to be altered.
		 *
		 * @since 2.3.0
		 *
		 * @param array $data Data for the export.
		 */
		$data = apply_filters( 'fes_export_get_data_' . $this->export_type, $data );

		return $data;
	}

	/**
	 * Gets the export data in EDD 3.0.
	 *
	 * @since 2.6.6
	 * @return array
	 */
	private function get_data_30() {
		$data       = array();
		$query_args = array(
			'no_found_rows' => true,
			'number'        => 99999,
		);

		// Export customers of a specific product.
		if ( ! empty( $_POST['edd_export_download'] ) ) {

			$query_args['product_id'] = absint( $_POST['edd_export_download'] );
			if ( isset( $_POST['edd_price_option'] ) ) {
				$query_args['price_id'] = (int) $_POST['edd_price_option'];
			}
		} else {
			$vendor   = new FES_Vendor( $_REQUEST['id'] );
			$products = EDD_FES()->vendors->get_all_products( $vendor->user_id );
			if ( empty( $products ) ) {
				return $data;
			}

			$query_args['product_id__in'] = wp_list_pluck( $products, 'ID' );
		}

		$order_items = edd_get_order_items( $query_args );

		if ( ! $order_items ) {
			return $data;
		}

		foreach ( $order_items as $order_item ) {
			$data[] = $this->get_order_item_data( $order_item );
		}

		return $data;
	}

	/**
	 * Gets the array of data from the order item.
	 *
	 * @since 2.6.6
	 * @param object $order_item \EDD\Orders\Order_Item
	 * @return array
	 */
	private function get_order_item_data( $order_item ) {
		$order    = edd_get_order( $order_item->order_id );
		$customer = edd_get_customer( $order->customer_id );

		return array(
			'product_id'    => $order_item->product_id,
			'product_title' => $order_item->product_name,
			'option_id'     => ! empty( $order_item->price_id ) ? $order_item->price_id : __( 'N/A', 'edd_fes' ),
			'option_title'  => ! empty( $order_item->price_id ) ? edd_get_price_option_name( $order_item->product_id, $order_item->price_id ) : __( 'N/A', 'edd_fes' ),
			'name'          => $customer->name,
			'email'         => $order->email,
			'date'          => $order_item->date_created,
		);
	}

	/**
	 * Gets the export data in EDD 2.x
	 * Originally in the get_data method; moved here for 3.0 compatibility.
	 *
	 * @since 2.6.6
	 * @return array
	 */
	private function get_data_29() {
		$data = array();
		if ( ! empty( $_POST['edd_export_download'] ) ) {

			// Export customers of a specific product
			global $edd_logs;
			$post = get_post( $_POST['edd_export_download'] );

			$args = array(
				'post_parent' => absint( $_POST['edd_export_download'] ),
				'log_type'    => 'sale',
				'nopaging'    => true
			);

			if ( isset( $_POST['edd_price_option'] ) ) {
				$args['meta_query'] = array(
					array(
						'key'   => '_edd_log_price_id',
						'value' => (int) $_POST['edd_price_option']
					)
				);
			}

			$logs = $edd_logs->get_connected_logs( $args );

			if ( $logs ) {
				foreach ( $logs as $log ) {
					$payment_id  = get_post_meta( $log->ID, '_edd_log_payment_id', true );
					$user_info   = edd_get_payment_meta_user_info( $payment_id );
					$cart_details = edd_get_payment_meta_cart_details( $payment_id );
					$option_id    = __( 'N/A', 'edd_fes' );
					$option_title = __( 'N/A', 'edd_fes' );
					foreach ( $cart_details as $cart_index => $download ) {
						if ( $download['id'] != $_POST['edd_export_download'] ){
							continue;
						}
						$download_type = edd_get_download_type( $download['id'] );
						$id      = isset( $download['item_number']['options']['price_id'] ) ? (int) $download['item_number']['options']['price_id'] : false;
						if ( $id ){
							$option_title = edd_get_price_option_name( $download['id'], $id );
							$option_id    = $id;
							break;
						}
					}

					$data[] = array(
						'product_id'    => $post->ID,
						'product_title' => $post->post_title,
						'option_id'     => $option_id,
						'option_title'  => $option_title,
						'name'          => $user_info['first_name'] . ' ' . $user_info['last_name'],
						'email'         => $user_info['email'],
						'date'          => $log->post_date
					);
				}
			}

		} else {
			$vendor = new FES_Vendor( $_REQUEST['id'] );
			$products = EDD_FES()->vendors->get_all_products( $vendor->user_id );
			$arr = array();
			if ( empty ( $products ) ){
				return;
			}
			foreach( $products as $product ){
				$arr[] = $product['ID'];
			}

			foreach( $arr as $product ){
				// Export customers of a specific product
				global $edd_logs;
				$post = get_post( $product );
				$args = array(
					'post_parent' => absint( $product ),
					'log_type'    => 'sale',
					'nopaging'    => true
				);

				$logs = $edd_logs->get_connected_logs( $args );

				if ( $logs ) {
					foreach ( $logs as $log ) {
						$payment_id   = get_post_meta( $log->ID, '_edd_log_payment_id', true );
						$user_info    = edd_get_payment_meta_user_info( $payment_id );
						$cart_details = edd_get_payment_meta_cart_details( $payment_id );

						$option_id    = __( 'N/A', 'edd_fes' );
						$option_title = __( 'N/A', 'edd_fes' );
						foreach ( $cart_details as $cart_index => $download ) {
							if ( $download['id'] != $product ){
								continue;
							}
							$download_type = edd_get_download_type( $download['id'] );
							$id      = isset( $download['item_number']['options']['price_id'] ) ? (int) $download['item_number']['options']['price_id'] : false;
							if ( $id ){
								$option_title = edd_get_price_option_name( $download['id'], $id );
								$option_id    = $id;
								break;
							}
						}
						$data[] = array(
							'product_id'    => $product,
							'product_title' => $post->post_title,
							'option_id'     => $option_id,
							'option_title'  => $option_title,
							'name'          => $user_info['first_name'] . ' ' . $user_info['last_name'],
							'email'         => $user_info['email'],
							'date'          => $log->post_date
						);
					}
				}
			}
		}

		return $data;
	}
}
