<?php
/**
 * Vendor Report: Earnings Over Time
 *
 * @package   edd-fes
 * @copyright Copyright (c) 2021, Sandhills Development, LLC
 * @license   GPL2+
 * @since     2.6.7
 */

class FES_Vendor_Earnings_Over_Time {

	/**
	 * @var array[]
	 */
	private $results = array(
		'sales'    => array(),
		'earnings' => array()
	);

	/**
	 * @var bool
	 */
	private $is_commissions_active = false;

	/**
	 * @var FES_Vendor|false
	 */
	private $vendor = false;

	/**
	 * @var array
	 */
	private $product_ids = array();

	/**
	 * @var array|\EDD\Utils\Date[]
	 */
	private $query_dates;

	/**
	 * @var bool
	 */
	private $day_by_day;

	/**
	 * @var bool
	 */
	private $hour_by_hour;

	/**
	 * FES_Vendor_Earnings_Over_Time constructor.
	 *
	 * @since 2.6.7
	 */
	public function __construct() {
		if ( EDD_FES()->integrations->is_commissions_active() ) {
			$this->is_commissions_active  = true;
			$this->results['commissions'] = array();
		}

		$this->set_vendor();

		$this->query_dates  = EDD\Reports\get_dates_filter( 'objects' );
		$this->day_by_day   = EDD\Reports\get_dates_filter_day_by_day();
		$this->hour_by_hour = EDD\Reports\get_dates_filter_hour_by_hour();
	}

	/**
	 * Sets the vendor to query.
	 *
	 * @since 2.6.7
	 */
	private function set_vendor() {
		$vendor_id = EDD\Reports\get_filter_value( 'fes_vendors' );
		if ( empty( $vendor_id ) ) {
			return;
		}

		$vendor = new FES_Vendor( $vendor_id );
		if ( empty( $vendor->id ) ) {
			return;
		}

		$this->vendor = $vendor;
	}

	/**
	 * Whether or not we have data to actually query.
	 *
	 * @since 2.6.7
	 * @return bool
	 */
	public function has_data_to_query() {
		if ( $this->vendor instanceof FES_Vendor && ! empty( $this->vendor->user_id ) ) {
			$user_ids = array( $this->vendor->user_id );
		} else {
			$vendors_db     = new FES_DB_Vendors();
			$vendors        = $vendors_db->get_vendors( array(
				'number' => 999999999999
			) );
			$user_ids = ! empty( $vendors ) && is_array( $vendors ) ? wp_list_pluck( $vendors, 'user_id' ) : array();
		}

		if ( empty( $user_ids ) ) {
			return false;
		}

		$this->product_ids = get_posts( array(
			'nopaging'    => true,
			'author__in'  => $user_ids,
			'orderby'     => 'title',
			'post_type'   => 'download',
			'post_status' => 'any',
			'order'       => 'ASC',
			'fields'      => 'ids'
		) );

		if ( empty( $this->product_ids ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Builds the SQL string to use for product IDs.
	 *
	 * @since 2.6.7
	 *
	 * @return string
	 */
	private function get_product_id_sql() {
		global $wpdb;

		$product_id_string = implode( ', ', array_fill( 0, count( $this->product_ids ), '%d' ) );

		return $wpdb->prepare( $product_id_string, $this->product_ids );
	}

	/**
	 * Queries for earnings and sales.
	 *
	 * @param bool $groupby Whether or not to group by date.
	 *
	 * @since 2.6.7
	 * @return object[]|object
	 */
	public function query_earnings( $groupby = true ) {
		global $wpdb;

		$product_id_string = $this->get_product_id_sql();
		$group_by_sql      = $groupby
			? "GROUP BY DATE(o.date_created) ORDER BY DATE(o.date_created)"
			: '';

		$query = $wpdb->prepare(
			"SELECT COUNT(o.id) AS sales, SUM(oi.total * oi.quantity) AS earnings, o.date_created AS date
 				 FROM {$wpdb->edd_orders} o
				 INNER JOIN {$wpdb->edd_order_items} oi ON(oi.order_id = o.id)
 				 WHERE o.date_created >= %s AND o.date_created <= %s
 				 AND o.type = 'sale'
 				 AND oi.status IN('complete', 'revoked')
				 AND oi.product_id IN({$product_id_string})
				 {$group_by_sql}",
			$this->query_dates['start']->copy()->format( 'mysql' ),
			$this->query_dates['end']->copy()->format( 'mysql' )
		);

		if ( $groupby ) {
			return $wpdb->get_results( $query );
		} else {
			return $wpdb->get_row( $query );
		}
	}

	/**
	 * Queries for commission earnings.
	 *
	 * @param bool $groupby Whether or not to group by date.
	 *
	 * @since 2.6.7
	 * @return object[]|object
	 */
	public function query_commissions( $groupby = true ) {
		if ( ! $this->is_commissions_active ) {
			return array();
		}

		global $wpdb;

		$table_name        = edd_commissions()->commissions_db->table_name;
		$product_id_string = $this->get_product_id_sql();
		$group_by_sql      = $groupby
			? "GROUP BY DATE(date_created) ORDER BY DATE(date_created)"
			: '';

		$query = $wpdb->prepare(
			"SELECT SUM(amount) AS earnings, date_created AS date
				FROM {$table_name}
				WHERE download_id IN({$product_id_string})
				AND status IN('paid', 'unpaid')
				AND date_created >= %s AND date_created <= %s
				{$group_by_sql}",
			$this->query_dates['start']->copy()->format( 'mysql' ),
			$this->query_dates['end']->copy()->format( 'mysql' )
		);

		if ( $groupby ) {
			return $wpdb->get_results( $query );
		} else {
			return $wpdb->get_row( $query );
		}
	}

	/**
	 * Retrieves the graph data.
	 *
	 * @since 2.6.7
	 * @return array|array[]
	 */
	public function get_earnings() {
		if ( ! $this->has_data_to_query() ) {
			return $this->results;
		}

		$order_results      = $this->query_earnings();
		$commission_results = $this->query_commissions();

		while ( strtotime( $this->query_dates['start']->copy()->format( 'mysql' ) ) <= strtotime( $this->query_dates['end']->copy()->format( 'mysql' ) ) ) {
			$timestamp = strtotime( $this->query_dates['start']->copy()->format( 'mysql' ) );

			$this->results['sales'][ $timestamp ][0] = $timestamp;
			$this->results['sales'][ $timestamp ][1] = 0;

			$this->results['earnings'][ $timestamp ][0] = $timestamp;
			$this->results['earnings'][ $timestamp ][1] = 0;

			if ( $this->is_commissions_active ) {
				$this->results['commissions'][ $timestamp ][0] = $timestamp;
				$this->results['commissions'][ $timestamp ][1] = 0;
			}

			foreach ( $order_results as $result ) {
				try {
					$this->add_row_to_results( $result, $timestamp, function ( $db_row, $timestamp ) {
						$this->results['sales'][ $timestamp ][1]    += $db_row->sales;
						$this->results['earnings'][ $timestamp ][1] += $db_row->earnings;
					} );
				} catch ( \Exception $e ) {

				}
			}

			foreach ( $commission_results as $result ) {
				try {
					$this->add_row_to_results( $result, $timestamp, function ( $db_row, $timestamp ) {
						$this->results['commissions'][ $timestamp ][1] += $db_row->earnings;
					} );
				} catch ( \Exception $e ) {

				}
			}

			// Move the chart along to the next hour/day/month to get ready for the next loop.
			if ( $this->hour_by_hour ) {
				$this->query_dates['start']->addHour( 1 );
			} elseif ( $this->day_by_day ) {
				$this->query_dates['start']->addDays( 1 );
			} else {
				$this->query_dates['start']->addMonth( 1 );
			}
		}

		return array_map( 'array_values', $this->results );
	}

	/**
	 * Formats the supplied database row and adds the values to the overall results.
	 *
	 * @param object  $db_row    Row from the database.
	 * @param int     $timestamp Current timestamp we're working with.
	 * @param Closure $callback  Callback function to execute for each condition.
	 *
	 * @since 2.6.7
	 * @throws Exception
	 */
	private function add_row_to_results( $db_row, $timestamp, $callback ) {
		$timezone         = new DateTimeZone( 'UTC' );
		$date_of_db_value = new DateTime( $db_row->date, $timezone );
		$date_on_chart    = new DateTime( $this->query_dates['start'], $timezone );

		// Add any sales/earnings that happened during this hour.
		if ( $this->hour_by_hour ) {
			// If the date of this db value matches the date on this line graph/chart, set the y axis value for the chart to the number in the DB result.
			if ( $date_of_db_value->format( 'Y-m-d H' ) === $date_on_chart->format( 'Y-m-d H' ) ) {
				call_user_func( $callback, $db_row, $timestamp );
			}
			// Add any sales/earnings that happened during this day.
		} elseif ( $this->day_by_day ) {
			// If the date of this db value matches the date on this line graph/chart, set the y axis value for the chart to the number in the DB result.
			if ( $date_of_db_value->format( 'Y-m-d' ) === $date_on_chart->format( 'Y-m-d' ) ) {
				call_user_func( $callback, $db_row, $timestamp );
			}
			// Add any sales/earnings that happened during this month.
		} else {
			// If the date of this db value matches the date on this line graph/chart, set the y axis value for the chart to the number in the DB result.
			if ( $date_of_db_value->format( 'Y-m' ) === $date_on_chart->format( 'Y-m' ) ) {
				call_user_func( $callback, $db_row, $timestamp );
			}
		}
	}

}
