<?php
/**
 * Vendor Reports
 *
 * @package   edd-fes
 * @copyright Copyright (c) 2021, Sandhills Development, LLC
 * @license   GPL2+
 * @since     2.6.7
 */

use EDD\Reports\Data\Report_Registry;

/**
 * Registers a new report filter for "Vendors".
 *
 * @param array $filters
 *
 * @since 2.6.7
 * @return array
 */
function fes_register_report_filters( $filters ) {
	$filters['fes_vendors'] = array(
		'label'            => __( 'Vendors', 'edd_fes' ),
		'display_callback' => 'fes_display_vendors_reports_filter'
	);

	return $filters;
}

add_filter( 'edd_report_filters', 'fes_register_report_filters' );

/**
 * Displays the "Vendors" filter dropdown.
 * We've built this out in case we want to one day integrate it with main reporting.
 * But for now, it's actually invisible in the one place it's actually used -- the
 * individual vendor profile.
 *
 * @since 2.6.7
 */
function fes_display_vendors_reports_filter() {
	if ( ! function_exists( 'EDD\\Reports\\get_filter_value' ) ) {
		return;
	}

	$screen = function_exists( 'get_current_screen' ) ? get_current_screen() : false;

	if ( $screen instanceof WP_Screen && 'toplevel_page_fes-vendors' === $screen->id ) {
		?>
		<input type="hidden" name="fes_vendors" value="<?php echo esc_attr( EDD\Reports\get_filter_value( 'fes_vendors' ) ); ?>">
		<?php
		return;
	}

	$vendors_db     = new FES_DB_Vendors();
	$vendors        = $vendors_db->get_vendors( array(
		'number' => 999999999999
	) );
	$vendor_options = array();

	if ( ! empty( $vendors ) && is_array( $vendors ) ) {
		foreach ( $vendors as $vendor ) {
			$vendor_name = ! empty( $vendor->email ) ? $vendor->email : $vendor->username;

			$vendor_options[ $vendor->id ] = $vendor_name;
		}
	}
	?>
	<span class="edd-graph-filter-options graph-option-section">
		<label for="edd_fes_vendors_filter" class="screen-reader-text">
			<?php esc_html_e( 'Filter by vendor', 'edd_fes' ); ?>
		</label>
		<?php
		echo EDD()->html->select( array(
			'options'          => $vendor_options,
			'name'             => 'fes_vendors',
			'id'               => 'edd_fes_vendors_filter',
			'selected'         => EDD\Reports\get_filter_value( 'fes_vendors' ),
			'show_option_all'  => __( 'All Vendors', 'edd_fes' ),
			'show_option_none' => false,
			'chosen'           => true
		) );
		?>
	</span>
	<?php
}

/**
 * Registers reports in EDD 3.0.
 *
 * @param Report_Registry $reports
 *
 * @since 2.6.7
 */
function fes_register_reports( $reports ) {
	try {
		$is_commissions_active = EDD_FES()->integrations->is_commissions_active();

		$tiles = array(
			'fes_vendor_total_earnings',
			'fes_vendor_total_sales',
		);
		if ( $is_commissions_active ) {
			$tiles[] = 'fes_vendor_total_commissions';
		}

		$reports->add_report( 'fes_vendors', array(
			'label'     => __( 'Vendor Earnings', 'edd_fes' ),
			'icon'      => 'index-card',
			'priority'  => 50,
			'filters'   => array( 'dates', 'fes_vendors' ),
			'endpoints' => array(
				'tiles'  => $tiles,
				'charts' => array(
					'fes_vendor_earnings'
				),
			),
		) );

		$reports->register_endpoint( 'fes_vendor_total_earnings', array(
			'label' => __( 'Total Earnings', 'edd_fes' ),
			'views' => array(
				'tile' => array(
					'data_callback' => function () {
						$graph    = new FES_Vendor_Earnings_Over_Time();
						$earnings = 0;

						if ( $graph->has_data_to_query() ) {
							$data     = $graph->query_earnings( false );
							$earnings = isset( $data->earnings ) ? $data->earnings : 0;
						}

						return edd_currency_filter( edd_format_amount( $earnings ) );
					},
					'display_args'  => array(
						'comparison_label' => '',
					)
				)
			)
		) );

		$reports->register_endpoint( 'fes_vendor_total_sales', array(
			'label' => __( 'Total Sales', 'edd_fes' ),
			'views' => array(
				'tile' => array(
					'data_callback' => function () {
						$graph = new FES_Vendor_Earnings_Over_Time();
						$sales = 0;

						if ( $graph->has_data_to_query() ) {
							$data  = $graph->query_earnings( false );
							$sales = isset( $data->sales ) ? (int) $data->sales : 0;
						}

						return $sales;
					},
					'display_args'  => array(
						'comparison_label' => '',
					)
				)
			)
		) );

		if ( $is_commissions_active ) {
			$reports->register_endpoint( 'fes_vendor_total_commissions', array(
				'label' => __( 'Total Commissions', 'edd_fes' ),
				'views' => array(
					'tile' => array(
						'data_callback' => function () {
							$graph       = new FES_Vendor_Earnings_Over_Time();
							$commissions = 0;

							if ( $graph->has_data_to_query() ) {
								$results     = $graph->query_commissions( false );
								$commissions = isset( $results->earnings ) ? $results->earnings : 0;
							}

							return edd_currency_filter( edd_format_amount( $commissions ) );
						},
						'display_args'  => array(
							'comparison_label' => '',
						)
					)
				)
			) );
		}

		$graph_datasets = array(
			'sales'    => array(
				'label'                => __( 'Sales', 'edd_fes' ),
				'borderColor'          => 'rgb(252,108,18)',
				'backgroundColor'      => 'rgba(252,108,18,0.2)',
				'fill'                 => true,
				'borderDash'           => array( 2, 6 ),
				'borderCapStyle'       => 'round',
				'borderJoinStyle'      => 'round',
				'pointRadius'          => 4,
				'pointHoverRadius'     => 6,
				'pointBackgroundColor' => 'rgb(255,255,255)',
			),
			'earnings' => array(
				'label'                => __( 'Earnings', 'edd_fes' ),
				'borderColor'          => 'rgb(24,126,244)',
				'backgroundColor'      => 'rgba(24,126,244,0.05)',
				'fill'                 => true,
				'borderWidth'          => 2,
				'type'                 => 'currency',
				'pointRadius'          => 4,
				'pointHoverRadius'     => 6,
				'pointBackgroundColor' => 'rgb(255,255,255)',
			),
		);
		if ( $is_commissions_active ) {
			$graph_datasets['commissions'] = array(
				'label'                => __( 'Commissions', 'edd_fes' ),
				'borderColor'          => 'rgb(27, 171, 49)',
				'backgroundColor'      => 'rgba(27, 171, 49,0.05)',
				'fill'                 => true,
				'borderWidth'          => 2,
				'type'                 => 'currency',
				'pointRadius'          => 4,
				'pointHoverRadius'     => 6,
				'pointBackgroundColor' => 'rgb(255,255,255)',
			);
		}

		$reports->register_endpoint( 'fes_vendor_earnings', array(
			'label' => __( 'Earnings Over Time', 'edd_fes' ),
			'views' => array(
				'chart' => array(
					'data_callback' => function () {
						$graph = new FES_Vendor_Earnings_Over_Time();

						return $graph->get_earnings();
					},
					'type'          => 'line',
					'options'       => array(
						'datasets' => $graph_datasets,
					),
				),
			),
		) );
	} catch ( EDD_Exception $e ) {
		edd_debug_log_exception( $e );
	}
}

add_action( 'edd_reports_init', 'fes_register_reports' );
